% Copyright 2018 John S Glover
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%
% 1. Redistributions of source code must retain the above copyright notice,
%    this list of conditions and the following disclaimer.
%
% 2. Redistributions in binary form must reproduce the above copyright notice,
%    this list of conditions and the following disclaimer in the documentation
%    and/or other materials provided with the distribution.
%
% 3. Neither the name of the copyright holder nor the names of its
%    contributors may be used to endorse or promote products derived from
%    this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
% NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
% CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
% STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
% ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

\documentclass[a4paper]{report}
\newif\ifshowcode
\showcodetrue

\newcommand{\docTitle}{Printer Filter for HP LJ1100}
\newcommand{\docAuthor}{Dr John S Glover}
\newcommand{\docCopyright}{\copyright~John Sigsworth Glover 2018}

\usepackage{a4wide}
\usepackage{latexsym}
%\usepackage{html}

\usepackage{listings}
\lstdefinelanguage{none}{
  identifierstyle=
}
\lstset{extendedchars=true, keepspaces=true, language=sh}

\usepackage{color}
\definecolor{linkcolor}{rgb}{0, 0, 0.7}

\usepackage[%
backref,%
raiselinks,%
pdfhighlight=/O,%
pagebackref,%
hyperfigures,%
breaklinks,%
colorlinks,%
pdfpagemode=None,%
pdfstartview=FitBH,%
linkcolor={linkcolor},%
anchorcolor={linkcolor},%
citecolor={linkcolor},%
filecolor={linkcolor},%
menucolor={linkcolor},%
pagecolor={linkcolor},%
urlcolor={linkcolor}%
]{hyperref}

\hypersetup{
  pdftitle={\docTitle},
  pdfsubject={FreeBSD printer filter driver},
  pdfauthor={\docAuthor},
  pdfkeywords={printer filter, Laserjet 1100, FreeBSD}
}

\setlength{\parskip}{1.5ex}

\usepackage{fancyhdr}
\pagestyle{fancyplain}
\fancyfoot{} % clear the footer
\fancyfoot[RO]{\thepage}
\fancyfoot[LO]{\docCopyright}
\renewcommand{\footrulewidth}{0.4pt}
\fancyhead{} % clear the header
\fancyhead[RO]{v1.1}
\fancyhead[LO]{\leftmark}
\renewcommand{\headrulewidth}{0.4pt}

\renewcommand{\chaptermark}[1]{
\markboth{\thechapter.\ #1}{}}

% for book and report style need to define the plain
% page style for oper
\fancypagestyle{plain}{%
\fancyhf{} % clear all header and footer
\fancyfoot[RO]{\thepage}
\fancyfoot[LO]{\docCopyright}
\renewcommand{\headrulewidth}{0pt}
\renewcommand{\footrulewidth}{0.4pt}}

\title{\docTitle}
\date{1 July 2018}
\author{\docAuthor}

\begin{document}
\pagenumbering{roman}
\maketitle
\tableofcontents

\ifshowcode

\chapter{Introduction}
\pagenumbering{arabic}

This printer driver is for the HP LaserJet 1100, a basic (but effective)
black and white laser printer.
The printer driver itself has been produced to run under FreeBSD, but
may work as is or with limited porting on other BSD based UNIX operating
systems.

\section{Prerequisites}
\label{sec:prerequisites}

This printer driver has a few prerequisites and dependencies:
\begin{description}
  \item[Ghostscript] a PostScript \cite{ASI1991} and PDF
  language interpreter and previewer.
  \item[a2ps] a utility which format files for printing on a
    PostScript printer.
  \item[nuweb] a simple literate programming tool.
    \verb|nuweb| is not required for the driver itself but to be able to
    update the \verb|ljfilt.w| source file.
    \verb|nuweb| can be obtained from \\
    \url{https://sourceforge.net/projects/nuweb/}\footnote{As of May~2018.}
  \item[\LaTeX] a structured text formatting and typesetting for \TeX used
    to reproduce this documentation.
\end{description}

\section{Abbreviations}

\begin{tabular}{|l|l|}
  \hline
  \textbf{Abbreviation} & \textbf{Description} \\
  \hline
  HP  & Hewlett Packard \\
  PCL & Printer control language \\
  PDF & Portable document format \\
  PS  & PostScript \\
  \hline
\end{tabular}

\chapter{Printer Filter}

The printer filter is a shell script, \verb|ljfilt|.

@o ljfilt
@{#!/bin/sh
#
# ljfilt, an HP LJ1100 printer filter supporting PostScript and PDF formats
#
@< BSD 3-clause licence @>
#
# Parts of this script were inspired by
# Swecoin's TTP8200 Unix drivers (http://www.swecoinus.com).
# and
# gsprint.sh Copyright (c) Olivier Mehani <shtrom-skb@ssji.net>, 2004
# see http://shtrom.ssji.net/skb/lpps.html
#
  
@< Define the filter configuration @>
@< Define the commands for ljfilt @>
@< Define the arguments for ghostscript @>
@< Create temporary copy of file to be printed @>
@< Check file type and print @>
@< Tidy up temporary file @>
# end of file
@}

The commands used by the filter are defined explicitly to avoid inadvertently
using a command or application in the search path.
However, there is a risk that the filter is fragile if the paths to
commands or installed applications are changed.

@d Define the commands for ljfilt
@{CAT=/bin/cat
RM=/bin/rm
COLRM=/usr/bin/colrm
HEAD=/usr/bin/head
LOGGER=/usr/bin/logger
MKTEMP=/usr/bin/mktemp
SED=/usr/bin/sed
A2PS=/usr/local/bin/a2ps
GS=/usr/local/bin/gs
@}

The file to be printed is replicated to a temporary file for subsequent
processing.

@d Create temporary copy of file to be printed
@{TEMPFILE=`$MKTEMP /tmp/ljfilt.XXXXXX`
if [ $? -ne 0 ]; then
	$LOGGER -t ljfilt "Error: couldn't create temporary file"
	exit 1
fi

# send stdin to the temporary file
$CAT > $TEMPFILE
@}

The filter accommodates three types of files:
\begin{itemize}
  \item PostScript, for which the file starts \verb|%!|;
  \item Portable document format (PDF), for which the file begins \verb|%PDF|;
  \item Plain text files.
\end{itemize}

The known file types (PostScript or PDF) are tested for and the appropriate
application used for the file type.
Failing the determination of a specific file type is supplied then the file
type is assumed to be plain text and processed accordingly.

@d Check file type and print 
@{# Test leading characters to determine the file's type
if [ "`$HEAD -1 $TEMPFILE | $COLRM 3`" = "%!" ]; then
	# PostScript file
	$GS $GSARGS $TEMPFILE
elif [ "`$HEAD -1 $TEMPFILE | $COLRM 5`" = "%PDF" ]; then
	# PDF file
	$GS $GSARGS $TEMPFILE
else
	# none of the above, we assume it is a plain text file
	@< Process plain text options @>
fi
@}

Ghostscript is used by the printer driver to translate from either PostScript
or PDF to HP PCL understood by the printer.
A number of arguments are defined for ghostscript:
\begin{description}
  \item \verb|-q| starts ghostscript silently without the usual startup
  messages.
  \item \verb|-sDEVICE| specifies an initial output device rather than the
  default, in this case that specified by \verb|GSDEVICE|.
  \item \verb|-dNOPAUSE| disables the prompt and pause at the end of each page.
  \item \verb|-sOutputFile| specifies a file for the output, in this case
    the standard output \verb|-|.
\end{description}

@d Define the arguments for ghostscript
@{GSARGS="-q -sDEVICE=$GSDEVICE -dNOPAUSE -dBATCH -sOutputFile=-"
@| GSARGS @}

The configuration of the filter is defined as follows:
@d Define the filter configuration
@{# gs driver to use
GSDEVICE=ljet4

# specify plain text file processing to be applied
TEXTFILTER=TEXTCRLF
@| GSDEVICE TEXTFILTER
@}

Plain text files can be processed in several ways, specified by
\verb|TEXTFILTER|, which is set manually in the printer driver.
\begin{description}
  \item \verb|TEXTA2PS| use the A2PS application to process file.
  \item \verb|TEXTCRLF| replace LF with a CRLF in the file.
  \item \verb|TEXTRAW| do nothing to the file.
\end{description}

@d Process plain text options
@{if [ "$TEXTFILTER" == "TEXTA2PS" ]; then
	$A2PS $TEMPFILE -o - | $GS $GSARGS -
elif [ "$TEXTFILTER" == "TEXTCRLF" ]; then
	# add a CR to the end of line
	CR=$'\r'
	$CAT $TEMPFILE | $SED -e "s/$/${CR}/g"
else
	# no filtering
	$CAT $TEMPFILE
fi@}

Having completed processing the temporary file for printing, tidy up by
deleting the temporary file created.
@d Tidy up temporary file
@{# cleanup
$RM $TEMPFILE
@}

\chapter{Installation}
For the installation of this filter, the following assumptions have been made:
\begin{itemize}
  \item User has \verb|root| access.
  \item Prerequisites installed, \S\ref{sec:prerequisites}.
\end{itemize}

\section{Installing the ljfilt script}

The installation process is currently a manual process.
Refer to the \S9 FreeBSD Handbook, \cite{FBSDDP2018}.

For FreeBSD, the printer filter, \verb|ljfilt|, should be installed in
\verb|/usr/local/libexec|.

Update the permissions and owner, as \verb|root|,

\begin{lstlisting}[language=none]
$ chown root wheel /usr/local/libexec/ljfilt

$ chmod 755 /usr/local/libexec/ljfilt
\end{lstlisting}

\section{Configuring printcap}

Based on the \S9.5.3 FreeBSD Handbook guidance, an example \verb|printcap| file
calling the filter \verb|ljfilt| is as follows:

\begin{lstlisting}[language=none]
lp:\
	:lp=/dev/lpt0:\
	:sh:\
	:mx#0:\
	:sd=/var/spool/lpd/lp:\
	:if=/usr/local/libexec/ljfilt:\
	:lf=/var/log/lpd-errs:
\end{lstlisting}

\section{Literate Programming}

\verb|ljfilt| is developed using literate programming techniques and the
\verb|nuweb|~\cite{Nuweb2018} tool.
\verb|nuweb| is inspired by the principles espoused by
Knuth~\cite{knuth1984literate}.

This filter is developed in a series of \verb|nuweb| files, \verb|*.w|,
which are processed to produce the target printer filter \verb|ljfilt|,
\verb|Makefile| and the documentation \LaTeX \verb|*.tex| file.

\section{Known issues}

\subsection{Security}

As this printer filter creates a copy of the file to be printed in
\verb|/tmp| these will be viewable by anyone with \verb|root| access
until they are deleted by the filter.
In addition, should the filter terminate before completing then a copy
of the file to be printed will remain.
Therefore, this printer filter may not be appropriate for secure or
sensitive tasks.

\subsection{a2ps Papersize}

When printing text files using \verb|TEXTA2PS| option they may fail
to print because the papersize for the printer has not been specified.
The papersize is detailed in \verb|${PREFIX}/etc/papersize|.
The default system papersize can be set using \verb|paperconfig|.

\chapter{Licence and Copyright}

This software uses the BSD 3-clause licence \cite{UniRegentsCalifornia1999}.
Both the printer filter, \verb|ljfilt|, and the \verb|Makefile| can use
the same text as they use the same comment nomenclature, \verb|#|.

@d BSD 3-clause licence
@{# Copyright 2018 John S Glover
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.@}

% Add the section on Makefile
@i ljfilt_make.w

\chapter{Indices} \label{indices}

Three sets of indices can be created automatically:
\begin{itemize}
  \item an index of file names, \S\ref{sec:files};
  \item an index of fragment names, \S\ref{sec:fragments};
  \item an index of user-specified identifiers, \S\ref{sec:identifiers}.
\end{itemize}

An index entry includes the name of the entry, where it
was defined, and where it was referenced.

\section{Files}
\label{sec:files}

@f

\section{Fragments}
\label{sec:fragments}

@m

\section{Identifiers}
\label{sec:identifiers}

@u

\fi

\bibliographystyle{plain}
\bibliography{ljfilt}

\end{document}
